#!/bin/bash

###############################################################################
printf -- '-%.0s' {1..80}
echo
echo "build-docker-compose-file.sh: Tool to bulid docker-compose.yaml file"
printf -- '-%.0s' {1..80}
echo

###############################################################################
# Remove tmp directory and recreate one for storing temp files
rm -rf ./tmp
mkdir ./tmp

###############################################################################
# Define functions for building docker containers

# Build a proxy docker container for the give proxy.
#
# Usage:
#   `build_proxy_docker protocol block_domains_cmds host port username password` OR
#   `build_proxy_docker protocol block_domains_cmds host port`
build_proxy_docker() {
    protocol="$1"
    block_domains_cmds="$2"
    host="$3"
    port="$4"
    username="${5-UNDEFINED}"
    password="${6-UNDEFINED}"
    outname="./tmp/tun${host}.yaml"

    if [[ ("$username" == "UNDEFINED") && ($password == "UNDEFINED") ]]; then
        proxy_string="$protocol://$host:$port"
        echo -e "${GREEN} BUILD PROXY DOCEKR: $protocol://$host:$port ${NOCOLOUR}" >&2
    else
        proxy_string="$protocol://$username:$password@$host:$port"
        echo -e "${GREEN} BUILD PROXY DOCEKR: $protocol://***:****@$host:$port ${NOCOLOUR}" >&2
    fi

    # Note the `proxy_string` contains special character ('/'),
    # we have to use another delimitor ('|' instead of '/') in sed
    sed -e "s/{{host}}/$host/g" \
        -e "s|{{proxy_string}}|${proxy_string}|g" \
        -e "s/{{block_domains_cmds}}/$block_domains_cmds/g" \
        ./templates/tun2socks_template.yaml >"$outname"

    # return the created temp file
    echo "$outname"
}

# Build packetshare docker container that "links" to the corresponding
# proxy docker via the network.
#
# The idea is that, the whole trafic to this docker container will be
# redirected through the proxy container, so that packetshare recogines
# the public IP of the running instance as the IP of the proxy.
#
# Usage:
#   `build_packetshare_docker proxy_host packetshare_email packetshare_password`
build_packetshare_docker() {
    proxy_host="$1"
    packetshare_email="$2"
    packetshare_password="$3"
    outname="./tmp/packetshare${proxy_host}.yaml"

    echo -e "${GREEN} BUILD PACKETSHARE DOCKER: $packetshare_email runing on host $proxy_host ${NOCOLOUR}" >&2
    sed -e "s/{{proxy_host}}/$proxy_host/g" \
        -e "s/{{packetshare_email}}/$packetshare_email/g" \
        -e "s/{{packetshare_password}}/$packetshare_password/g" \
        ./templates/packetshare_template.yaml >"$outname"

    # return the created temp file
    echo "$outname"
}

# Build one proxy container and one packetshare container for each proxy.
#
# The definition of these 2 containers are placed together in a docker
# compose file, which will be run afterward to create docker containers.
#
# Usage:
#   `build_docker_compose_file proxy_file packetshare_email packetshare_password block_domains_cmds`
build_docker_compose_file() {
    compose_file="./docker-compose.yaml"

    # Remove existing file if exists
    rm -rf $compose_file

    # Prepare file structure
    echo -e "name: nga-proxy-linux-packetshare\n" >>$compose_file
    echo -e "services:\n" >>$compose_file

    # Input params
    proxy_file="$1"
    packetshare_email="$2"
    packetshare_password="$3"
    block_domains_cmds="$4"

    if [ ! -f "$proxy_file" ]; then
        echo -e "${RED}Proxy file $proxy_file does not exist ${NOCOLOUR}" >&2
        exit 1
    fi

    # Remove special characters from proxy file
    sed -i 's/\r//g' "$proxy_file"

    # Read proxy file line-by-line and build one container for each proxy
    proxy_count=0
    while IFS= read -r proxy_line || [ -n "$proxy_line" ]; do
        line=$(trim "$proxy_line")
        if [[ -z "$line" ]]; then
            continue
        fi

        # Split proxy text string into params
        # Support format: `host:port:[username:[password]]` (meaning that the username/password are optional)
        IFS=':' read -r -a proxy_params <<<"$line"
        ip_proxy_host="${proxy_params[0]}"

        proxy_tmp_file=$(
            # For now only support http protocol in the proxy string
            build_proxy_docker "http" "$block_domains_cmds" "${proxy_params[@]}"
        )
        packetshare_tmp_file=$(
            build_packetshare_docker "$ip_proxy_host" "$packetshare_email" "$packetshare_password"
        )

        cat "$proxy_tmp_file" >>$compose_file
        cat "$packetshare_tmp_file" >>$compose_file

        # Clean temp files
        rm -rf "$proxy_tmp_file"
        rm -rf "$packetshare_tmp_file"

        # Store the current processed proy to the temp file to be used for delay start
        echo "$ip_proxy_host" >>./tmp/processed_proxy.txt
        ((proxy_count += 1))

    done <"$proxy_file"

    echo -e ">>>>${GREEN} [$proxy_count] proxies processed.${NOCOLOUR}\n" >&2

    # Return the docker compose file name
    echo $compose_file
}
