#!/bin/bash
set -o errexit -o pipefail -o noclobber -o nounset +x

###############################################################################
# variables for global files
PROPERTY_FILE="property.conf"
PROXY_FILE="proxy.txt"
BLACKLIST_FILE="blacklist.txt"

# global variables for packetshare info, to be load from property file
PACKETSHARE_EMAIL=
PACKETSHARE_PASSWORD=

###############################################################################
# We use `getopt` enhanced version, it is expected that the --test give error 4.
# Let's run getopt and ignore errexit with `&& true`
getopt --test >/dev/null && true
if [[ $? -ne 4 ]]; then
    echo -e "{RED} Please update linux kernel version to use getopt enhanced version" >&2
    exit 1
fi

# options that require 1 argument: --property_file/-t, --proxy_file/-p, --blacklist_file/-b
LONGOPTS=quick,dry,property_file:,proxy_file:,blacklist_file:,delay:,help
OPTIONS=qdt:p:b:l:h

# Guide for parsing arguments: https://stackoverflow.com/a/29754866/11722995
# -temporarily store output to be able to check for errors
# -activate quoting/enhanced mode (e.g. by writing out “--options”)
# -pass arguments only via   -- "$@"   to separate them correctly
# -if getopt fails, it complains itself to stderr
PARSED=$(getopt --options=$OPTIONS --longoptions=$LONGOPTS --name "$0" -- "$@") || exit 2
# read getopt’s output this way to handle the quoting right:
eval set -- "$PARSED"

DRY_RUN_MODE=false
QUICK_RUN_MODE=false
DELAY_SECOND=

# now enjoy the options in order and nicely split until we see --
while true; do
    # echo "DEBUG PARAM: $1, $@ ($#)"
    case "$1" in
    -q | --quick)
        QUICK_RUN_MODE=true
        shift
        ;;
    -d | --dry)
        DRY_RUN_MODE=true
        shift
        ;;
    -t | --property_file)
        PROPERTY_FILE="$2"
        shift 2
        ;;
    -p | --proxy_file)
        PROXY_FILE="$2"
        shift 2
        ;;
    -b | --blacklist_file)
        BLACKLIST_FILE="$2"
        shift 2
        ;;
    -l | --delay)
        DELAY_SECOND="$2"
        shift 2
        ;;
    -h | --help)
        cat help.txt
        exit 0
        ;;
    --)
        shift
        break
        ;;
    *)
        echo "Programming error: Wrong parameters. See '--help'"
        exit 3
        ;;
    esac
done

###############################################################################
# Show tool title
echo >&2
cat title.txt
echo >&2

echo "Run the program with these following options:
    DRY MODE: $DRY_RUN_MODE
    QUICK MODE: $QUICK_RUN_MODE
    PROPERTY_FILE: $PROPERTY_FILE
    PROXY_FILE: $PROXY_FILE
    BLAKLIST_FILE: $BLACKLIST_FILE

" >&2

# read -p "Confirm that if it is correct? Yes(Y/y) No(N/n) " -n 1 -r
# echo >&2
# if [[ ! ($REPLY =~ ^[Yy]$) ]]; then
#     echo "Terminated without running anything." >&2
#     exit 0
# fi

# Load util functions and other global variables for coloring text
# shellcheck source=utils.sh
source utils.sh

###############################################################################
# Load property file and check if the needed params are loaded
load_properties "$PROPERTY_FILE"
echo "Packetshare account: $PACKETSHARE_EMAIL"
if [[ (! $PACKETSHARE_EMAIL) || (! $PACKETSHARE_PASSWORD) ]]; then
    echo -e "${RED} Cannot load Packetshare Email/Password ${NOCOLOUR}"
    exit 1
fi

# Load the blacklist to process the domains to be blocked
if [[ ! ("$BLACKLIST_FILE" == "NONE") ]]; then
    block_domains_cmds=$(build_iptables_blacklist_cmds "$BLACKLIST_FILE")
else
    block_domains_cmds=""
fi

###############################################################################
# Check if we need to re-run all
if [[ $QUICK_RUN_MODE == true ]]; then
    echo -e "QUICK MODE: Do not rebuild docker compose file\n" >&2
    docker_compose_file='./docker-compose.yaml'
else
    # Load the functions used to build docker containers
    # shellcheck source=build-docker-compose-file.sh
    source build-docker-compose-file.sh

    # Before building the containers, pre-pull the tun2socks and packetshare docker images
    echo -e "${GREEN} \nPrepare docker images ......... ${NOCOLOUR}" >&2
    docker image pull --quiet xjasonlyu/tun2socks:v2.5.2
    docker image pull --quiet packetshare/packetshare

    # Build the docker compose file
    echo -e "${GREEN} \nBuilding docker containers ......... ${NOCOLOUR}" >&2
    docker_compose_file=$(
        build_docker_compose_file "$PROXY_FILE" "$PACKETSHARE_EMAIL" "$PACKETSHARE_PASSWORD" "$block_domains_cmds"
    )
fi

###############################################################################
if [[ $DRY_RUN_MODE == true ]]; then
    echo -e "DRY MODE: Check the docker-compose.yaml file.\nDONE\n" >&2
    exit 0
fi

# Create and run the docker container using `docker compose`
# Make sure the docker compose exist
if ! [ -f "${docker_compose_file}" ]; then
    echo -e "${RED} 'docker-compose.yaml' does not exist in the current directory." >&2
    echo -e "${RED} Re-run the tool WITHOUT '--quick' option.'" >&2
    exit 1
fi

echo -e "${GREEN} \nRunning docker containers ......... ${NOCOLOUR}" >&2

if [[ "$DELAY_SECOND" ]]; then
    echo -e "Applying delay of $DELAY_SECOND second(s) between the start of containers.\n" >&2
    start_container_with_delay "$DELAY_SECOND" ./tmp/processed_proxy.txt
else
    docker compose -f "$docker_compose_file" up -d
fi

echo -e "${GREEN} \nUse 'docker compose stats' to monitor the running containers.\n\nDONE\n\n ${NOCOLOUR}" >&2
