#!/bin/bash

# Colours
# shellcheck disable=SC2034  # Variables used in other scipts
RED="\033[0;31m"
GREEN="\033[0;32m"
YELLOW="\033[0;33m"
NOCOLOUR="\033[0m"

# Remove whitespace from the input string.
# Ref: https://stackoverflow.com/a/3352015/11722995
trim() {
    local var="$*"
    # remove leading whitespace characters
    var="${var#"${var%%[![:space:]]*}"}"
    # remove trailing whitespace characters
    var="${var%"${var##*[![:space:]]}"}"
    printf '%s' "$var"
}

# Read the properties file and export variables to the current shell
load_properties() {
    property_file="${1}"
    echo -e "\nLoading properties from $property_file" >&2

    # Read property file line-by-line (deal with file has 'no terminal newline')
    while IFS= read -r line || [ -n "$line" ]; do
        # Ignore lines that start with #
        if [[ $line != '#'* ]]; then
            # Split the line at the first occurrence of =
            key="${line%%=*}"
            value="${line#*=}"
            # Trim leading and trailing whitespace from key and value
            key="${key%"${key##*[![:space:]]}"}"
            value="${value%"${value##*[![:space:]]}"}"
            # Ignore lines without a value after =
            if [[ -n $value ]]; then
                # Replace variables with their values
                value=$(eval "echo $value")
                # Export the key-value pairs as variables
                export "$key"="$value"
            fi
        fi
    done <"$property_file"
}

# Build iptables command to drop traffic comming from any domain in the given blacklist_file
#
# Usage:
#   build_iptables_blacklist_cmds blacklist_file
#
# Return: the iptables commands (separated by ';') concatenated into a string
build_iptables_blacklist_cmds() {
    blacklist_file="$1"
    echo -e "\nProcessing blacklist file $blacklist_file" >&2

    block_commands=""
    block_count=0

    # Read blacklist file line-by-line, (deal with file has 'no terminal newline')
    while IFS= read -r block_line || [ -n "$block_line" ]; do
        # Ignore empty lines or lines that start with #
        line=$(trim "$block_line")
        if [[ $line && $line != '#'* ]]; then
            # See the DEV note for more details about the iptables string matching
            block_commands+=" iptables -A INPUT -m string --string '$line' --algo kmp -j DROP; "
            block_commands+=" iptables -A FORWARD -m string --string '$line' --algo kmp -j DROP; "
            ((block_count += 1))
        fi

    done <"$blacklist_file"

    echo -e ">>>>${GREEN} [$block_count] domains will be blocked by iptables.${NOCOLOUR}\n" >&2

    # Return the created commands
    echo "$block_commands"
}

# Util function to start the containers with delay.
#
# The delay time in seconds is given in the first argument.
# The container name is determined from the input file as the second argument.
#
# Usage:
#   start_container_with_delay delay_in_second list_containers_file
start_container_with_delay() {
    delay_in_second="$1"
    list_containers_file="$2"

    if [[ ! -f $list_containers_file ]]; then
        echo -e "${RED} File containing list of containers ${list_containers_file} does not exist. ${NOCOLOUR}" >&2
        exit 1
    fi

    while IFS= read -r ip_proxy_host || [ -n "$ip_proxy_host" ]; do
        docker compose up -d "tun2socks-${ip_proxy_host}"
        docker compose up -d "packetshare-${ip_proxy_host}"
        sleep "$delay_in_second"
    done <"$list_containers_file"
}
